#ifndef AKENTI_CERTIFICATE_IMPL_H
#define AKENTI_CERTIFICATE_IMPL_H

#include "AkentiIssuer.hpp"
#include "UtcTime.hpp"
#include "Xerces-Decls.hpp"

namespace Akenti {

class PublicKey;
class PrivateKey;

/**
  * This abstract class is the base for all the akenti certificates
  * implementations. 
  *
  * <p>
  * We have designed all the certificates to be immutable. 
  * Subclass Implementors should adhere to this policy.                <BR>
  *
  *  1) The set methods should be accessed by the genarators only.     <BR> 
  *
  *  2) The doReadObject should be protected.                          <BR>
  *
  * <p>
  * Encoding Rules:                                                    <BR>
  *   1) The various fields are separated by one blank space.          <BR>
  *
  *   2) The <i>AkentiOutputStream</i> is configured                   <BR> 
  *      to write primitive types and user define types.               <BR>
  *
  * <p>
  * Note:                                                              <BR>
  *   The encoding is straightforward with the exception 
  *   of the version number which is preceded by
  *   the character 'V'.
  *
  * <pre>
  * <b>CertificateEncoding</b> {
  *    type             // certificate's type 
  *    version          // certificate's version number
  *    id               // certificate's unique id 
  *    issuer           // issuer and signer of certificate     
  *    notBefore        // start validity date 
  *    notAfter         // end validity date 
  *    canonAlg         // name of algorithm used to format certificate
  *                        for signing or pem encoding.
  *    signatureAlg     // name of algorithm used to sign this certificate
  *    ...              // fields contained in the concrete subclasses
  *    signature        // digital signature of the certificate
  *  }
  * </pre>
  *
  *
  * @see AkentiCertGenerator
  * @see AkentiCertificate
  * @see AkentiOutputStream
  * @see AkentiInputStream
  * @see AkentiObject
  * @see AkentiIssuer
  * @see UtcTime
  *
  * @version 1.1 00/04/17
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari 
  **/

class AkentiCertificateImpl {

public:

  /**
    * Destroys this object.
    */
  virtual ~AkentiCertificateImpl();

  /**
    * Computes the md5 hashcode of this <i>AkentiCertificateImpl</i> 
    * object for searching purposes.
    *
    * @return the hashcode of this AkentiCertificateImpl
    * @see getHashString
    */
  virtual string hashCode() const;

  /**
    * Returns the approriate hash string whose digest will be used to
    * simplify certificate retrieval process.
    *
    **/
  virtual string getHashString() const = 0;

  /**
    * Returns the type of this <i>AkentiCertificateImpl</i> object.
    *
    * @return the type of this AkentiCertificateImpl
    */
  const string& getType() const;
 
  /**
    * Returns <i>true</i> if this certificate's type
    * is equal to the argument specified by <i>type</i>.
    */
  bool hasType(const string& type) const;

  /**
    * Returns the version number of this <i>AkentiCertificateImpl</i> object. 
    *
    * @return the version number of this AkentiCertificateImpl
    */
  int getVersionNumber() const;

  /**
    * Returns the unique id of this <i>AkentiCertificateImpl</i> object. 
    *
    * @return the unique id of this AkentiCertificateImpl 
    **/
  const string& getUniqueID() const;

  /**
    * Returns the issuer of this <i>AkentiCertificateImpl</i> object.
    * 
    * @return the issuer of this AkentiCertificateImpl 
    *
    * @see AkentiPrincipal
    */
  const AkentiIssuer& getIssuer() const;

  /**
    * Returns the start validity date of this 
    * <i>AkentiCertificateImpl</i> object. 
    *
    * @return the start validity date of this AkentiCertificateImpl 
    *
    * @see UtcTime
    **/
  const UtcTime& getNotBefore() const;

  /**
    * Returns the end validity date of this 
    * <i>AkentiCertificateImpl</i> object. 
    *
    * @return the end validity date of this AkentiCertificateImpl 
    *
    * @see UtcTime
    **/
  const UtcTime& getNotAfter() const;

  /**
    * Verifies the signature of <i>AkentiCertificateImpl</i> object
    * using a <i>PublicKey</i> specified by <i>publicKey</i>.
    *
    * @param publicKey used to verufy signature
    * @return true if signature verifies 
    *
    * @see PublicKey
    **/
  bool verify(const PublicKey& publicKey) const;

  /**
    * Returns the signable portion of this <i>AkentiCertificateImpl</i>
    * object.
    *
    * @return the signable portion of this AkentiCertificateImpl
    *
    * @see sign(string, PrivateKey)
    **/
  virtual string getSignable() const;

  /**
    * Returns the name of the formatting algorithm that was used 
    * for the signature or pem encoding.
    *
    * @return the name of the formatting algorithm
    */
  const string& getCanonAlg() const;

  /**
    * Returns the signature algorithm that was used to 
    * sign this <i>AkentiCertificateImpl</i> object.
    *
    * @return the name of the signature algorithm 
    *
    * @see sign(string, PrivateKey)
    **/
  const string& getSignatureAlg() const;

  /**
    * Returns the signature that resulted from signing 
    * the signable part of this <i>AkentiCertificateImpl</i>
    * object.
    *
    * @return the signature of this AkentiCertificateImpl 
    *
    * @see getSignable() 
    * @see sign(string, PrivateKey)
    **/
  const string& getSignature() const;

  /**
    * Is this <i>AkentiCertificateImpl</i> valid at the time
    * specified by <i>utcTime</i>?
    *
    * @return true if notBefore <= utcTime <= notAfter
    *
    * @see getNotBefore()
    * @see getNotAfter()
    * @see UtcTime
    **/
   bool checkValidity(const UtcTime& utcTime) const;

  /**
    * Is this <i>AkentiCertificateImpl</i> valid now?
    *
    * @return true if notBefore <= now <= notAfter
    *
    * @see getNotBefore()
    * @see getNotAfter()
    * @see UtcTime
    **/
  bool checkValidity() const;

  /**
    * Returns debugging info.
    *
    * Subclass implementors must implement the pure virtual 
    * function <i>doParamString(ostream&)</i> so that the 
    * additional params are included.
    *
    * @return string a string representation of this object
    **/
  string paramString() const;

  /**
    * Returns the Akenti Certificate in text format.
    * 
    * @param indentAmt the number of spaces to indent the text form of the 
    *   Akenti Certificate
    */
  string toText(int indentAmt = 0) const;

  /**
    * Returns logging info.
    */
  string toLogString() const;

private:

  friend class AkentiCertGenerator;       // for acces to set methods
  friend class AkentiCertificate;         // for access to readObject and writeObject
  friend class AkentiCertificateFactory;  // for access to newInstance

  /**
    * Sets the version number of this <i>AkentiCertificateImpl</i> object
    * given an <i>int</i> specified by <i>version</i>.
    *
    * @param version a version number 
    *
    * @see getVersionNumber()
    */
  void setVersionNumber(int version);

  /**
    * Sets the name of the formatting algorithm used when
    * signing or pem encoding this <i>AkentiCertificateImpl</i>.
    *
    * @param canonAlg name of algorithm.
    */
  void setCanonAlg(const string& canonAlg);
  
  /**
    * Sets the unique id of this <i>AkentiCertificateImpl</i> object
    * given a <i>string</i> specified by <i>id</i>.
    *
    * @param id a unique id
    */
  void setUniqueID(const string& id);
  
  /**
    * Sets the issuer of this <i>AkentiCertificateImpl</i> object
    * given an <i>AkentiPrincipal</i> specified <i>issuer</i>.
    *
    * @param issuer issuer 
    *
    * @see getIssuer
    * @see AkentiPrincipal
    */
  void setIssuer(const AkentiIssuer& issuer);

  /**
    * Sets the start validity date of this <i>AkentiCertificateImpl</i> object 
    * given a <i>UtcTime</i> specified by <i>notBefore</i>.
    *
    * @param notBefore start validity date 
    *
    * @see getNotBeforte()
    * @see UtcTime
    */
  void setNotBefore(const UtcTime& notBefore);

  /**
    * Sets the end validity date of this <i>AkentiCertificateImpl</i> object 
    * given a <i>UtcTime</i> specified by <i>notAfter</i>.
    *
    * @param notAfter the end validity date 
    *
    * @see getNotAfter()
    * @see UtcTime
    */
  void setNotAfter(const UtcTime& notAfter);

  /**
    * Signs this <i>AkentiCertificateImpl</i> object using the
    * signature alogorithm specified by <i>signatureAlg</i> and
    * the <i>PrivateKey</i> specified by <i>privateKey</i>.
    * 
    * @param signatureAlg name of algorithm to be used
    * @param privateKey private key used for signing
    * 
    * @see getSignable()
    * @see PrivateKey
    */ 
   virtual void sign(const string& signatureAlg, const PrivateKey &privateKey);

  /**
    * Reads this object from the <i>AkentiInputStream</i> object
    * specified by <i>source</i>. 
    *
    * Subclass implementors must implement the pure virtual 
    * function <i>doReadObject(AkentiOutputStream&)</i>
    * so that the additional fields in the subclass get written.
    *
    * @param source AkentiInputStream used to read this object
    * @exception IOException if reading fails
    * @exception ParsingException if the parsing of the distinguished name fails
    *
    * @see doWriteObject(AkentiInputStream&) 
    * @see AkentiInputStream
    * @see IOException
    * @see ParsingException
    **/
  virtual void readObject(AkentiInputStream& source);

  /**
    * Writes this object to an <i>AkentiOutputStream</i> object
    * specified by <i>drain</i>.
    *
    * Subclass implementors must implement the pure virtual 
    * function <i>doWriteObject(AkentiInputStream&)</i>
    * so that the additional fields in the subclass get written.
    *
    * @param drain AkentiOutputStream used to write this object
    * @exception IOException if writing fails
    *
    * @see doWriteObject(AkentiInputStream&) 
    * @see AkentiOutputStream
    * @see IOException
    */
  virtual void writeObject(AkentiOutputStream &drain) const;

  /**
    * Internal function.
    *
    * Writes this object to the <i>AkentiOutputStream</i> object
    * specified by <i>drain</i>.
    * 
    * if the flag <i>includeSignature</i> is set to <i>true</i>
    * then the signature info is also written.
    *
    */
  void writeObject(AkentiOutputStream& drain, bool includeSignature) const; 

  /**
    */
  void toXML(DOMElement* element) const;
  
  /**
    */
  void fromXML(const DOMElement* element); 

protected:

  /**
    * Constructs this object given a <i>string</i> specified
    * by <i>type</i>.
    */
  AkentiCertificateImpl(const string& type = "NO_TYPE");

  /**
    */
  virtual void doWrite(AkentiOutputStream& drain) const = 0;

  /**
    */
  virtual void doRead(AkentiInputStream& source) = 0;

  /**
    */
  virtual void doToXML(DOMElement* element) const {
  }
  
  /**
    */
  virtual void doFromXML(const DOMElement* element) {
  }

  /**
    * Returns an instance of the concrete class. 
    */
  virtual AkentiCertificateImpl* newInstance() const = 0;

  /**
    */
  virtual void doParamString(ostream& os) const = 0;

  /**
    */
  virtual void doToText(int indentAmt, ostream& os) const = 0;

protected:

  string type;

  int version;

  string id;

  AkentiIssuer issuer;

  UtcTime notBefore;

  UtcTime notAfter;

  string canonAlg;

  string signatureAlg;

  string signature;

  string hashcode;

};

} // end of name space

#endif
