#ifndef CERTIFICATE_AUTHORITY_INFO_HPP
#define CERTIFICATE_AUTHORITY_INFO_HPP

#include <string>
#include <vector>
#include "AkentiObject.hpp"
#include "DistinguishedName.hpp"
#include "Certificate.hpp"
#include "Directory.hpp"

using std::string; 
using std::vector; 

namespace Akenti {

/**
  * A <i>CertificateAuthorityInfo</i> holds ca's chain of
  * x509 certs. This chain can consist solely of the ca's
  * cert.   
  *
  * In addition, although not required, directories where
  * to search for identity certs and crls issued by the ca 
  * can be supplied.
  *
  * CertificateAuthorityInfo {                      <br>
  *   DistingushedName                    ca;       <br>
  *   vector&lt;Certificate&gt;        certs;       <br>
  *   vector&lt;Directory&gt;         idDirs;	    <br>
  *   vector&lt;Directory&gt;        crlDirs;	    <br>
  * }                                               <br>  
  *
  * The format of writeObject is following :                         <br>
  *   ca certCount cert+ idDirCount idDir* crlDirCount crlDir* <br>
  *
  * Where cert is the pem encoding of an x509 certificate,
  *       + means one or more
  *       * means 0 or more
  *
  * @see AkentiObject
  * @see Certificate 
  * @see Directory
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  * @author Xiang Sun
  **/

class CertificateAuthorityInfo : public AkentiObject {

public:

  /** 
    * Default constructor.
    **/
  CertificateAuthorityInfo();
  
  /**
    * Constructs this object given a ca's certificate specified by 
    * <i>cert</i>.
    *
    * @param cert    Certificate        the ca's certificate
    * @param idDirs  vector<Directory>  where to search for x509 certs
    * @param crlDirs vector<Directory>  where to search for crls
    *
    * @see DistinguishedName
    * @see Certificate
    **/   
  CertificateAuthorityInfo(const Certificate& cert, 
                           const vector<Directory>& idDirs = vector<Directory>(),
                           const vector<Directory>& crlDirs = vector<Directory>());

  /**
    * Constructs this object given a ca's chain of certificates 
    * specified by certs. Note that the last cert in the chain
    * is considered to belong to the ca.
    *
    * @param certs vector<Certificate>  a ca's chain of x509 certs
    * @param idDirs  vector<Directory>  where to search for x509 certs
    * @param crlDirs vector<Directory>  where to search for crls
    *
    **/   
  CertificateAuthorityInfo(const vector<Certificate>& certs,
                           const vector<Directory>& idDirs = vector<Directory>(),
                           const vector<Directory>& crlDirs = vector<Directory>());
  /**
    * Deletes this object.
    **/
  ~CertificateAuthorityInfo();

  /**
    * Returns the distinguished name represented by this 
    * <i>CertificateAuthorityInfo</i> object.
    *
    * @return the distinguished name of the ca
    *
    * @see DistinguishedName
    **/
  const DistinguishedName& getCa() const;

  /**
    * Returns the ca's x509 cert chain. 
    *
    * @see Certificate
    */
  const vector<Certificate>& getChain() const;

  /**
    * Returns the ca's identity directories.
    *
    * @see Directory
    */
  const vector<Directory>& getIdentityDirectories() const; 

  /**
    * Returns the ca's CRL directories.
    *
    * @see Directory
    **/
  const vector<Directory>& getCrlDirectories() const; 

  /**
    * Writes this object to the <i>AkentiOutputStream</i>
    *
    * @param drain AkentiOutputStream used to write this object
    * @exception IOException if writing fails
    * @exception SecurityException if encoding of certificate fails
    * @exception Error if no security provider is installed
    *
    * @see AkentiOutputStream
    * @see SecurityException
    * @see IOException
    * @see Error
    **/
  void writeObject(AkentiOutputStream& drain) const;

  /**
    * Reads this object from the <i>AkentiInputStream</i>.
    *
    * @param source AkentiInputStream used to read this object
    * @exception IOException if reading from AkentiInputStream fails
    * @exception SecurityException if decoding of the certificate fails
    * @exception Error if no security provider is installed
    *
    * @see AkentiInputStream
    * @see SecurityException
    * @see IOException
    * @see Error 
    **/
  void readObject(AkentiInputStream& source);

  /**
    * Populates the XML element <i>caInfo</i> from this 
    * <i>CertificateAuthorityInfo</i>.
    */
  void toXML(DOMElement* caInfo) const;

  /**
    * Initializes this <i>CertificateAuthorityInfo</i> from the XML element
    * <i>caInfo</i>
    */
  void fromXML(const DOMElement* caInfo);

  /**
    * Used for debugging.
    *
    * @return a string representation of this object
    *
    * @see AkentiObject
    **/
  string paramString() const;

private:

  DistinguishedName ca;         // ca's dDistingushedName

  vector<Certificate> certs;   // ca's x509 chain

  vector<Directory> idDirs;    // where to find x509 certs 

  vector<Directory> crlDirs;   // where to find CRLs

};

} // end of name space

#endif
