#ifndef AKENTI_CERTIFICATE_CACHE_HPP
#define AKENTI_CERTIFICATE_CACHE_HPP

#include <string>
#include <vector>
#include <map>
#include "CertificateStatusListener.hpp"
#include "CacheCertificate.hpp"
#include "AKCertHolder.hpp"

using std::string;
using std::vector;
using std::map;
using namespace Akenti;

namespace Akenti {

/**
  * The <i>CertificateCache</i> caches certificates in memory.
  * The external cache is consulted only if no certificates
  * are stored internally. In addition this cache keeps track
  * of what is loaded from the external cache and will only 
  * update the external cache if the information that was loaded
  * has changed.
  * 
  * @see AKCertHolder
  * @see ExternalCache 
  * @see CertificateStatusListener
  *
  * @version 1.1 00/05/01
  * @author Abdlelilah Essiari
  * @author Srilekha Mudumbai
  */
class CertificateCache {

public:

  /**
    * Constucts this object.
    */
  CertificateCache();

  /**
    * destroys this object.
    */
  ~CertificateCache();

  /**
    * Sets the name of the directory where the certificates 
    * will be stored externally.
    */
  void 
  setDirectoryName(const string& rootResourceName);

  /**
    * Constructs a <i>AKCertHolder</i> using the input parameters
    * and adds it to the cache.
    *
    * @return a AKCertHolder that wraps the certificate
    * @see AKCertHolder
    */
  AKCertHolder&
  addCertificate(const AkentiCertificate& cert,
                 int level,
                 const UtcTime& startCacheTime,
                 const UtcTime& endCacheTime);

  AKCertHolder&
  addAKCertHolder(const AKCertHolder& holder);
  
  /**
    * Returns a vector of <i>AKCertHolder</i> holders that pass the 
    * following tests.
    * <pre>
    * 1) holder.cert->hashCode() == key.
    * 2) holder.cert->getType() == type.
    *
    * If the holders were just loaded from the external cache then the 
    * the certificate's validity is tested.
    *
    * Note that the external cache is queried only if the internal cache 
    * has no certificates that satisfy the two tests mentioned above.
    */
  vector<AKCertHolder>  
  getAKCertHolders(const string& key, const string& type);

  /**
    * Returns a vector of <i>AKCertHolder</i> holders that pass the 
    * following tests.
    * <pre>
    * 1) holder.cert->hashCode() == key.
    * 2) holder.cert->getType() == type.
    * 3) holder.startCacheTime >= notBefore.
    *
    * If the holders were just loaded from the external cache then the 
    * the certificate's validity is tested.
    *
    * Note that the external cache is queried only if the internal cache 
    * has no certificates that pass the first two tests.
    *</pre>
    */
  vector<AKCertHolder>  
  getAKCertHolders(const string& key, 
                   const string& type,
                   const UtcTime& notBefore);
  /**
    * Updates the external cache.
    *
    */
  bool 
  updateExternalCache();

private:

  CertificateCache(const CertificateCache&);

  void operator=(const CertificateCache&);

  CacheCertificate
  generateCacheCertificate(const vector<AKCertHolder>& holders);

  class Entry : public CertificateStatusListener {
  
  public:

    Entry();

    ~Entry();

    void setViewChanged(bool);

    bool getViewChanged() const;

    void setAKCertHolders(const vector<AKCertHolder>&);

    int containsCertificate(const AkentiCertificate&) const;

    int size() const;

    AKCertHolder& operator[](int);

    AKCertHolder& 
    addAKCertHolder(const AKCertHolder&);

    void
    certStartCacheTimeChanged(const AkentiCertificate&);

    void
    certEndCacheTimeChanged(const AkentiCertificate&);

    void
    verifyingCertOK(const AkentiCertificate&);

    void
    verifyingCertFailed(const AkentiCertificate&);

  private:

    vector<AKCertHolder> certHolders;

    bool viewChanged;

  };

  map<string, Entry> table;

  string rootResourceName;
  
  bool useExtCache;

};

} // end of name space

#endif
