#ifndef AK_PROVIDER_IMPL_HPP
#define AK_PROVIDER_IMPL_HPP 

#include <string>
#include "SSLConstants.hpp" 

using std::string;

namespace Akenti {

class MessageDigest;
class Signature;
class Base64;
class Hmac;
class Cipher;
class SecureSocketImpl;
class KeyStore;
class DistinguishedName;
class KeyFactory;
class CertificateFactory;
class SSLContext;

/**
  * A <i>ProviderImpl</i> object provides applications with engines that
  * implement security algorithms.
  *
  * Applications must install a provider at startup.
  *
  * @see Provider#setProvider(Provider&)
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class ProviderImpl {

public:

  /**
    * Deletes this object.
    */
  virtual ~ProviderImpl() {
  }

  /**
    * Loads <i>MessageDigest<i> algorithms.
    */
  virtual void loadDigestAlgorithms() = 0;

  /**
    * Returns <i>true</i> if <i>alg</i> is supported.
    *
    * @return true if alg is supported, false otherwise
    */
  virtual bool isDigestAlgorithmSupported(const string& alg) const = 0;

  /**
    * Returns a <i>MessageDigest</i> engine given the name of a standard
    * algorithm specified by <i>alg</i> such as MD5.
    *
    * For convenience , the <i>MessageDigest::getInstance()</i> can
    * be used.
    *
    * @param alg string standard name of an algorithm 
    *
    * @return a <i>MessageDigest</i> engine. 
    * @exception SecurityException if algorithm is not supported
    *
    * @see loadDigestAlgorithms()
    * @see isDigestAlgorithmSupported(const string&)
    * @see MessageDigest#getInstance(const string& alg)
    */ 
  virtual MessageDigest getMessageDigest(const string& alg) const = 0; 

  /**
    * Loads <i>Signature</i> algorithms.
    */
  virtual void loadSignatureAlgorithms() = 0;

  /**
    * Returns <i>true</i> if <i>alg</i> is supported.
    *
    * @return true if alg is supported, false otherwise
    */
  virtual bool isSignatureAlgorithmSupported(const string& alg) const = 0;

  /**
    * Returns a <i>Signature</i> engine given the name of a standard
    * algorithm specified by <i>alg</i> such as RSA-MD5 or RSA-SHA1. 
    *
    * For convenience , the <i>Signature::getInstance()</i> can
    * be used.
    *
    * @param alg string standard name of a <i>Signature</i> algorithm 
    *
    * @return a <i>Signature</i> engine 
    * @exception SecurityException if algorithm is not supported
    *
    * @see loadSignatureAlgorithms()
    * @see sSignatureAlgorithmSupported(const string&)
    * @see Signature#getInstance(const string& alg)
    */ 
  virtual Signature getSignature(const string& alg) const = 0; 

  /**
    * Loads <i>Cipher<i> algorithms.
    */
  virtual void loadCipherAlgorithms() = 0;

  /**
    * Returns <i>true</i> if <i>alg</i> is supported.
    *
    * @return true if alg is supported, false otherwise
    */
  virtual bool isCipherAlgorithmSupported(const string& alg) const = 0;

  /**
    * Returns a <i>Cipher</i> engine given the name of a standard
    * algorithm specified by <i>alg</i> such as DES-CBC or RC4
    *
    * For convenience, the <i>Cipher::getInstance()</i> can
    * be used.
    *
    * @param alg string standard name of a <i>Cipher</i> algorithm
    *
    * @return a <i>Cipher</i> engine 
    * @exception SecurityException if algorithm is not supported
    *
    * @see loadCipherAlgorithms()
    * @see isCipherAlgorithmSupported(const string&)
    * @see Cipher#getInstance(const string& alg)
    */
  virtual Cipher getCipher(const string& alg) const = 0;

  /**
    */
  virtual bool seedRandomGenerator(const string& seed) = 0;

  /**
    * Returns a <i>Base64</i> object. Applications need not call 
    * this function. Instead, the static method <i>Base64::getInstance()</i>
    * should be used.
    *
    * @return a <i>Base64</i> object
    * @see Base64#getInstance()
    * @see Base64Impl
    */
  virtual Base64 getBase64() const = 0;

  /**
    */
  virtual KeyStore getKeyStore(const string& type) const = 0;

  /**
    * Returns a <i>KeyFactory</i> object. Applications need not call this
    * function. Instead, the static method <i>KeyFactory::getInstance()</i>
    * should be used.  
    *
    * @return a <i>KeyFactory</i> object
    * @see KeyFactory#getInstance()
    * @see KeyFactoryImpl
    */ 
  virtual KeyFactory getKeyFactory() const = 0;

  /**
    * Returns a <i>CertificateFactory</i> object. Applications need not call 
    * this function. Instead, the static method 
    * <i>CertificateFactory#getInstance()</i> should be used.  
    *
    * @return a <i>CertificateFactory</i> object
    * @see CertificateFactory#getInstance()
    * @see CertificateFactoryImpl
    */ 
  virtual CertificateFactory getCertificateFactory() const = 0;
  
  /**
    */
  virtual Hmac getHmac(const string& alg) const = 0;
  
  /**
    * Create a either a server or client security context.
    * The concrete ProviderImpl can create whatever sort of security 
    * context it needs.
    *
    * @param context version number
    * @return A security context
    */
  virtual SSLContext 
  createSSLContext(SSLConstants::Version version, bool isClient) const = 0;

  /**
    * Interface to create a SecureSocketImpl. The concrete providerImpl should 
    * create the type of SecureSocketImpl it needs.
    */
  virtual SecureSocketImpl*
  createSecureSocketImpl(const SSLContext& ctx) const = 0;

  /**
    */
  virtual string toDerString(const DistinguishedName& dn) const = 0;

protected:

  /**
    * Constructs this object.
    */
  ProviderImpl() {
  }

};

} // end of namespace Akenti

#endif
