#ifndef AK_URL_HPP
#define AK_URL_HPP

#include <iosfwd>
#include <string>
#include <map>

using std::string;
using std::map;
using std::ostream;

namespace Akenti {

class URLHandler;
class URLConnection;

/**
  * URL class refers to Uniform Resource Locator and can be used to 
  * locate resources on the Internet. An example of a url is
  * <p>
  *   http://www-itg.lbl.gov/Akenti
  *
  * <p>
  *   A URL is comprised of the following:
  *   <br>
  *   <i> <b> protocol </b> - the protocol (http, file, ldap etc.) that
  *			      should be used <br>
  *   <i> <b> host </b> - the host to be connected to <br>
  *       <b> port </b> - the port to connect to <br>
  *       <b> file </b> - the file/information to be retrieved <br>
  *   </i>
  *
  * <p>
  *   In the above example, the protocol to use is <i> http </i> and the
  * host to connect is <i> www-itg.lbl.gov </i>. The port information is
  * not given and hence the default port is used. The file component
  * to be retrieved is <i> Akenti </i>. If a directory is given, an index
  * of files that belong to that directory will be displayed. If a default
  * file (for ex., index.html) is available under that directory, as
  * configured in the web server, then that file will be retrieved. You
  * must specify a filename if the file to be retrieved is different
  * from the default file in that directory.
  * 
  * <p>
  * The default port for the http protocol is 80. Any other port 
  * should be explicitly mentioned in the URL as follows <br>
  *
  *   http://imglib.lbl.gov:8080/downloads 
  * <p>
  *   There is no Anchor '#" support for this URL. Hence it will be considered
  * as a part of the file to be retrieved. This URL is a final class and
  * should not be extended.
  * <p>
  * The methods supported by this class involve setting or getting
  * componets of the URL name. The URL also contains a reference to a 
  * URLHandler which refers to a URLHandlerImpl which is a friend class to URL.
  * The URLHandlerImpl implements the methods to connect to a URL and retrieve
  * information from it.
  *
  * @see MalformedURLException
  * @see URLHandlerImpl
  * @see FileURLHandlerImpl
  * @see URLHandlerImpl
  * @see LdapURLHandlerImpl
  *
  * @version 1.1 07/19/00
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari 
  *
  */

class URL {

public:

  friend class URLHandlerImpl;

  /**
    * Constructs a null URL. Uses the defaults for a null URL:" noprot", 
    * "nohost", port=-1,"/nofile" and NullURLHandler.
    *
    */
  URL();

  /**
    * Constructs this object. Uses the URLHandlerFactory to create a
    * URLHandler for the specified protocol.
    * The default URLHandler for files is FileURLHandlerImpl,
    * for http is HttpURLHandlerImpl and for ldap is LdapURLHandlerImpl.
    *
    * @param url string containing the url.
    * @exception MalformedURLException if url cannot be parsed.
    */
  URL(const string& url);

  /**
    * Constructs this object.
    *
    * @param protocol any one of these protocols - http, file or ldap
    * @param host connect to this host
    * @param port the port to connect to the host for sending requests
    * @param file the information to be retrieved from the url
    */
  URL(const string& protocol, const string& host, int port, const string& file); 

  /**
    * Destructor
    */
  ~URL();

  /**
    * Returns a string containing the protocol of this url object
    *
    * @return protocol component of this url
    */
  const string& getProtocol() const;

  /**
    * Returns a string containing the host of this url object
    *
    * @return host component of this url
    */
  const string& getHost() const;

  /**
    * Returns an integer containing the port of this url object
    *
    * @return port component of this url
    */
  int getPort() const;

  /**
    * Returns a string containing the file name of the information to be
    * retrieved from this url.
    *
    * @return file component of this url
    */
  const string& getFile() const;

  /**
    * Returns the string representation of this url. See the class description
    * for the possible formats of URLs.
    * 
    * @exception 
    **/
  string toString() const;

  /**
    * Returns a URLConnection object associated with the URL. URLConnection
    * objects provide the methods to open and close connections to the URL
    * and read and write data through the connection.
    *
    * @return a URLConnection object for this URL.
    */

  URLConnection getURLConnection() const;

  /**
    */
   operator void*() const;

  /**
    */
  bool operator==(const URL& url) const;

  /**
    * Returns a string representation for URL, which is used  
    * debugging purposes.
    *
    * @return debugging info
    */
  string paramString() const;

private:
  
  /**
    * Sets the port number
    *
    * @param port the port number
    */
  void setPort(int port);

  /**
    * Sets the name of the host to which connection is made.  
    *   
    * @param host the name of the host 
    */
  void setHost(const string& host);

  /**
    * Sets the name of the file from which infromation is to be retrieved.
    *
    * @param host the name of the host
    */
  void setFile(const string& file);

  static URLHandler& getURLHandler(const string& protocol);

  string protocol;

  string host;

  int port;

  string file;

};

} // end namespace Akenti

ostream& operator<<(ostream& os, const Akenti::URL& url);

#endif
