#include <iostream> 
#include <string> 
#include <vector> 
#include <unistd.h>
#include "ProtocolHandler.hpp"
#include "CheckPolicyRequest.hpp"
#include "CheckPolicyResponse.hpp"
#include "CertificateFactory.hpp"
#include "CertificateChain.hpp"
#include "AkentiInit.hpp"
#include "CapabilityCertificate.hpp"
#include "AkentiMessage.hpp"
#include "File.hpp"
#include "Exception.hpp"
#include "Error.hpp"

using namespace Akenti;
using namespace std;

/**
  * <pre>
  * <b>CheckPolicy:</b> 
  *         Use this program to check a resource's policy.
  *
  * Usage:  CheckPolicy -r resourceName
  *                     [-f AkentiEngineConfigFile]
  *                     [-v]
  *                     [-h]
  * </pre>
  */

namespace CheckPolicy {

void printPurpose () {
  cerr << "Purpose: Use this program to check a resource policy." << endl
       << endl;
}

void printUsage () {
  cerr << "Usage: CheckPolicy args" << endl
       << "  -f arg       - akenti engine config file" << endl
       << "  -r arg       - resource" << endl
       << "  [-l arg]     - level 1, 2, or 3" << endl
       << "  [-a]         - use level 3" << endl
       << "  [-v]         - verbose" << endl
       << "  [-h]         - prints this message" << endl
       << "where each level is as follows:"  << endl
       << "1 - prints WORKED/FAILED only" << endl
       << "2 - prints all messages collected (this is the default level)" << endl
       << "3 - prints level 3 and all the certificates as well, same as -a option" << endl;
}

int main(int argc, char ** argv) {
  string confFileName;
  string resource;
  bool verbose = false;
  int level = 2; 
  int opt;

  try {
    if (argc < 2) {
      printPurpose();
      printUsage();
      exit(2);
    }

    while ((opt = getopt(argc,argv,"f:l:r:va")) != EOF) {
      switch (opt) {
        case 'f':
         confFileName = optarg;
         break;
       case 'l':
         level = atoi(optarg);
         break;
       case 'r':
         resource = optarg;
         break;
       case 'v':
         verbose = true;
         break;
       case 'a': 
         level = 3;
         break;
       case 'h':
       default:
         printUsage();
	 exit(2);
      }
    }

    if ((level < 1) || (level > 3)) {
        cerr << "Error: Level not defined" << endl;
        printUsage ();
        exit(2);
    }

    if (!resource.length()) {
	cerr << "no resource name given" << endl;
	printUsage();
	exit(2);
    }

    string status;

    if (verbose) { 
      cout << "Calling AkentInit::init with configuration file " << confFileName << endl;
    }

    File file(confFileName);

    if (!file.isFile()) {
      cerr << "Can't find config file" << endl;
      exit(2);
    } else if (!AkentiInit::init(confFileName, status)) {
      cerr << "AkentiInit::init failed. " << status << endl;
      exit(2);
    }

    if (verbose) {
      cout << "Status from AkentiInit: "<< status << endl; 
    }

    CheckPolicyRequest req;
    CheckPolicyResponse resp;

    req.setID(1);
    req.setResource(resource);
    ProtocolHandler::getInstance(ProtocolHandler::ENGINE).handle(req, resp);

    const vector <PolicyInfo>&  policyInfoVec = resp.getPolicyInfos();

    AkentiMessage mesg = resp.getAkentiMessage();

    bool worked = (mesg.getCode() == 150);

    cout << "\n----------------------" << endl;

    if (worked) {
       cout << "CheckPolicy WORKED" << endl;
    } else {
       cout << "CheckPolicy FAILED" << endl;
    }

    cout << "----------------------" << endl;

    if (level == 1) { // we are done
       exit(worked == false);
    }

    for (int j = 0; j < policyInfoVec.size(); ++j) {
        string resource = policyInfoVec[j].getResource();
        const vector <CertificateInfo>& certInfoVec = policyInfoVec[j].getCertificateInfos();

        for (int i = 0; i <certInfoVec.size();  ++i) {
           CertificateInfo certInfo = certInfoVec[i];
           cout << "----------------------" << endl;
    
           string type;
           if (certInfo.hasCertificate()) {
              type = (certInfo.getCertificate().get())->getType();
           }
           else {
              type = "dummy";
           }  
           
           cout << "Resource = " << resource << ", Used = " << certInfo.getUsed(); 
           if (type == "dummy") {
              cout << ", Top level error" << endl;
           } else {
               cout << ", Type = " << type << endl;
           }
           
           if (type == "UseCondition") {
              cout << "Location = " << certInfo.getLocation() << endl;
           }
    
           const vector<AkentiMessage>& mesgs = certInfo.getCertificateMessages();
        
           for (int j = 0; j < mesgs.size(); ++j) {
             cout << mesgs[j].getCode() << ": " << mesgs[j].getMessage() << endl;
           }
    
           if (level == 3) {
               if (type != "dummy") {
                   AkentiCertificate::writeToStream("XML", cout, certInfo.getCertificate());
               }
           }
    
           cout << "----------------------" << endl;
        }
  }
    exit(worked == false);
  } catch(Exception& ex) {
    cerr << "EXCEPTION:" << ex << endl;
    exit(2);
  } catch (Error& err) {
    cerr << "ERROR:Report to Akenti team:" <<  err << endl;
    exit(3);
  }
}

}
