#ifndef AK_KEY_PAIR_HPP
#define AK_KEY_PAIR_HPP

#include <string>
#include "PublicKey.hpp" 
#include "PrivateKey.hpp" 
#include "Error.hpp" 

using std::string;

namespace Akenti {

/**
  * This class is a simple holder for a key pair, a public key and 
  * a private key. It does not enforce any security and, when 
  * initialized, should be treated like a private key.
  *
  * @see PublicKey
  * @see PrivateKey
  * @see KeyFactory
  * @see KeyFactoryImpl#generateRSAKeyPair(int, unsigned long)
  * @see KeyFactoryImpl#generateDSAKeyPair(int, unsigned long)
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari 
  */

class KeyPair {

public:

  /**
    * Default Constructor.
    *
    */
  KeyPair() {
  }

  /**
    * Constructs a key pair given a public key and a private key.
    *
    * @param pubKey the public key.
    * @param privKey the private key.
    * @exception Error if eithey key is uninitialized
    * @see PublicKey
    * @see PrivateKey
    */
  KeyPair(const PublicKey &pubKey, const PrivateKey &privKey)
     : publicKey(pubKey), privateKey(privKey) {
    if (!pubKey || !privKey) {
      throw Error("KeyPair:constructor:invalid argument");
    }
  }

  /**
    * Destroys this object.
    * 
    * @see ref_ptr
    * @see PublicKey 
    * @see PrivateKey 
    */
  ~KeyPair() {
  }

  /**
    * Returns a reference to the private key component of this key pair.
    *
    * @return a reference to the private key 
    * @exception Error if key pair is uninitialzed
    * @see PrivateKey
    */
  const PrivateKey& getPrivateKey() const {
    if (!privateKey) {
      throw Error("KeyPair:getPrivateKey:invalid operation");
    }

    return privateKey;
  }

  /**
    * Returns a reference to the public key component of this key pair.
    *
    * @return a reference to the public key 
    * @exception Error if key pair is uninitialzed
    * @see PublicKey
    */
  const PublicKey& getPublicKey() const {
    if (!publicKey) {
      throw Error("KeyPair:getPublicKey:invalid operation");
    }

    return publicKey;
  }

  /**
    * Returns a string representation of this key pair for 
    * debugging purposes.
    * 
    * @exception Error if one of the keys is uninitialized
    * @return debugging info 
    */
  string paramString() const {
    string params;
    params += '[';
    if (!publicKey) { 
      params += "unitialized key pair"; 
    } else {
      params += "publicKey"; 
      params += '=';
      params += '"';
      params += publicKey->paramString();
      params += '"';
      params += ',';
      params += "privateKey"; 
      params += '=';
      params += '"';
      params += privateKey->paramString();
      params += '"';
    }
    params += ']';
    return params;
  }

private:

  PublicKey publicKey;     // public key component of this pair 

  PrivateKey privateKey;   // private key component of this pair

};

} // end of namespace Akenti

#endif
