#ifndef USE_COND_ISSUER_GROUP
#define USE_COND_ISSUER_GROUP

#include <string>
#include <vector>
#include "AkentiPrincipal.hpp"
#include "Directory.hpp"

using std::string;
using std::vector;

namespace Akenti {

/**
  * A <i>UseCondIssuerGroup</i> object consists of one more 
  * <i>AkentiPrincipal</i> who represent Use-Condition Certificate Issuers 
  * and one or more locators that are used to locate Use-Condition certificates
  * issued by those issuers.
  *
  * <pre>
  *
  * UseCondIssuerGroup { 
  *     vector&lt;AkentiPrincipal&gt;  issuers;   // Use-Condition certificate 
  *                                               //   issuers
  *     vector&lt;Directory&gt;         ucDirs;   // where to find these uc 
  *                                               //   certs.
  * }                                    
  *
  * The format of a UseCondIssuerGroup is: 
  *   issuerCount issuer+ ucDirCount ucDir+
  * </pre>
  *
  * @see AkentiObject
  * @see AkentiPrincipal
  * @see Directory 
  *
  * @version 1.1 01/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class UseCondIssuerGroup : public AkentiObject {

public:

  /**
    * Constructs this object.
    * 
    * @param issuers Use-Condition Certificate Issuers 
    * @param ucDirs  locations of the Use-Condition certificates 
    *
    * @see AkentiPrincipal 
    * @see Directory
    **/
  UseCondIssuerGroup(const vector<AkentiPrincipal>& issuers = vector<AkentiPrincipal>(1), 
                     const vector<Directory>& ucDirs = vector<Directory>(1));

  /**
    * Destroys this object.
    **/
  ~UseCondIssuerGroup();

  /**
    * Returns the Use-Condition Certificate issuers.
    * 
    * @see AkentiPrincipal
    **/
  const vector<AkentiPrincipal>& getUCIssuers() const;
  
  /**
    * Returs <i>true</i> if <i>issuer</i> is in the 
    * list of the Use-Condition certificate issuers. 
    *
    * @see AkentiPrincipal
    */
  bool containsUCIssuer(const AkentiPrincipal& issuer) const;

  /**
    * Returns the Use-Condition certificate directories.
    *
    * @see Directory 
    **/ 
  const vector<Directory>& getDirectories() const;

  /**
    * Writes this object to an <i>AkentiOutputStream</i>.
    *
    * @see AkentiOutputStream
    **/
  void writeObject(AkentiOutputStream& drain) const;

 /**
   * Reads this object from an <i>AkentiInputStream</i>.
   *
   * @see AkentiInputStream 
   **/
  void readObject(AkentiInputStream& source);

  /**
    * Populates the XML element <i>ucGroup</i> from this <i>UseCondIssuerGroup</i>.
    */
  void toXML(DOMElement* ucGroup) const;

  /**
    * Initializes this <i>UseCondIssuerGroup</i> from the XML element
    * <i>ucGroup</i>
    */
  void fromXML(const DOMElement* ucGroup);

  /**
    * Used for debugging.
    *
    **/
  string paramString() const;

private:

  vector<AkentiPrincipal> issuers;      // Use-Condition certificate issuers
  
  vector<Directory>     ucDirs;      // pointers to where these certificates are stored

};

} // end of name space

#endif
